"""
CLI utility that gives the ISO reporting week for a given date.
Option allows week ending day of Saturday or Sunday.
Option for week number of most recently completed week.
Example: wn --last
    2023-W48
"""

import pendulum
from pendulum.parsing.exceptions import ParserError
import typer
from typing_extensions import Annotated, Optional, List

app = typer.Typer()

def input_date_validate(value: str):
    try:
        pendulum.parse(value, strict=False)
    except ParserError as ex:
        raise typer.BadParameter(f"{ex}, try YYYY-MM-DD format.")
    return value

def today_datestring():
    return pendulum.now().to_date_string()

def week_number_string(date, sunday_weekend = False):
    if sunday_weekend:
        return date.strftime("%G-W%V")
    else:
        return date.add(days=1).strftime("%G-W%V")

def verbose_output(date, parsed_date, sunday_weekend, last_week):
    weekend_day = "Sunday" if sunday_weekend else "Saturday"

    if sunday_weekend:
        weekend_date = parsed_date.add(days=-1).next(pendulum.SUNDAY)
    else:
        weekend_date = parsed_date.add(days=-1).next(pendulum.SATURDAY)

    if last_week: weekend_date = weekend_date.subtract(weeks=1)

    typer.echo(f"Input date string: {date}")
    typer.echo(f"Timezone:          {pendulum.now().timezone.name}")
    typer.echo(f"Parsed date:       {parsed_date.to_date_string()}")
    typer.echo(f"Last week flag:    {last_week}")
    typer.echo(f"Week end day:      {weekend_day}")
    typer.echo(f"Week end date:     {weekend_date.to_date_string()}")


@app.command()
def main(
    date: Annotated[str
        , typer.Argument(
            callback=input_date_validate
            , help="A text expression of date, ex: 'November 27', or 2112-07-29. Default is today's in current TZ."
            , default_factory=today_datestring)]
    , sunday_weekend: Annotated[bool
        , typer.Option(
            "--sunday"
            , help="Week end is Saturday by default, this flag sets Sunday weekend day (ISO standard).")] = False
    , last_week: Annotated[bool
        , typer.Option(
            "--last"
            , help="Give week number of most recently completed week (overrides DATE argument).")] = False
    , verbose: Annotated[bool
        , typer.Option(
            "--verbose", "-v"
            , help="Full parsed date details for verification."
        )] = False
    ):
    """
    ISO year week number of a date as YYYY-"W"WW. Default weekend day is Saturday.\n
    Example:\n
    $> wn 'Jul 22 2020' --last\n
    2020-W29
    """
    parsed_date = pendulum.parse(date, strict=False)
    if verbose: verbose_output(date, parsed_date, sunday_weekend, last_week)

    if last_week:
        result = week_number_string(parsed_date.subtract(weeks=1), sunday_weekend=sunday_weekend)
    else:
        result = week_number_string(parsed_date, sunday_weekend=sunday_weekend)

    typer.echo(result)


# Pytest example: using temporary test directory
#
#   poetry run pytest --capture=no ./tests/test_unit.py
#   pytest --capture=no ./tests/test_unit.py

import pytest
import os

# test that pytest tmp directory is empty
# use the system tmp directory base in a test
def test_unit(tmp_path):
    tmp_dir = tmp_path
    system_temp = os.environ['TMPDIR']

    print("\n\tsystem tmp directory: {}".format(system_temp))
    print("\n\ttmp_path for this test run: {}".format(tmp_dir))

    assert len(os.listdir(tmp_dir)) == 0



"""A command line utility to create a symlinked folder in the current directory, pointing to the 'too' project root directory.

For the purpose of keeping source code and notes as text files, with associated documents, data, and images stored separately on a parallel file structure. The main benefits are browsing through directories and scanning the notes contents visually, and looking through filenames without a lot of clutter.

NOTE: git repos can be moved anywhere, because their path is uniquely definied by root name.

Usage:

too big|data|doc
"""
# TODO: [--verbose, -v] version print 
#   folder created: $HOME/toodoc/datasci/biq/biq-206/bigtest
#   symlink:        doc -> $HOME/toodoc/datasci/biq/biq-206/bigtest

from argparse import ArgumentParser
from git import Repo, exc
from pathlib import Path

POSITIONAL_ARGS = {
    'big': 'toobig'
    , 'data': 'toodata'
    , 'doc': 'toodoc'
    }

def cli_parser(args = None):
    if args: args = [args]       # must do for test calls
    arg_choices = list(POSITIONAL_ARGS)
    parser = ArgumentParser(
        prog = "too"
        , description = "Create symlinked parallel folders under HOME/{toobig|toodata|toodoc}, to contain data/binary files outside of\n git repo or away from source/text files."
        )
    parser.add_argument(
        'too_dir'
        , choices = arg_choices
        , help = "large files to exclude from backup, smallish datasets, binary files like pdf"
        )
    args = parser.parse_args(args)
    return args.too_dir

def is_git_repo(path):
    try:
        _ = Repo(path, search_parent_directories=True)
        return True
    except exc.InvalidGitRepositoryError:
        return False

def repo_root(repo_path):
    try:
        repo = Repo(repo_path, search_parent_directories=True)
        return Path(repo.working_tree_dir)
    except exc.InvalidGitRepositoryError:
        return None

def top_dir_path(tbdd):
    topname = POSITIONAL_ARGS.get(tbdd, "ERROR")
    return Path.home() / topname

def make_topdir_and_link(new_path):
    try:
        new_path.mkdir(parents=True, exist_ok=True)
    except:
        print("Could not make TooPath directory.")
    try:
        Path(TOPDIR_NAME).symlink_to(new_path)
    except FileExistsError:
        print("Symlink already exists")

def too_path(any_path):
    if is_git_repo(any_path):
        repo_root_path = repo_root(any_path)
        path_from_root = any_path.relative_to(repo_root_path)
        return top_dir_path(TOPDIR_NAME) / repo_root_path.name / path_from_root
    else:
        path_from_home = any_path.relative_to(Path.home())
        return top_dir_path(TOPDIR_NAME) / path_from_home


def main(too_dir = None):
    global TOPDIR_NAME

    too_dir = cli_parser(too_dir)
    TOPDIR_NAME = too_dir

    make_topdir_and_link(too_path(Path.cwd()))

if __name__ == "__main__":
    main()



# test command line arg handling, help function, and core functions

import pytest
import os
import git
from pathlib import Path
from desertislandutils.src.toobigdatadoc import too

def test_help(capsys):
    help_args = ["-h", "--help"]
    help_msg_part = "usage: too"

    for i in help_args:
        with pytest.raises(SystemExit) as pytest_wrapped_exception:
            too.main(i)
        assert pytest_wrapped_exception.type == SystemExit
        assert pytest_wrapped_exception.value.code == 0

        captured = capsys.readouterr()
        assert len(captured.err) == 0
        assert help_msg_part in captured.out

def test_bad_input(capsys):
    bad_input = "42 -h poo"
    expected_out = "too: error:"
    with pytest.raises(SystemExit) as pytest_wrapped_exception:
            too.main(bad_input)
    assert pytest_wrapped_exception.type == SystemExit
    captured = capsys.readouterr()
    assert expected_out in captured.err

def test_non_repo(tmp_path, monkeypatch):
    tmp_dir = tmp_path
    test_path_from_home = "non_repo_dir/subdir_1/subdir_2"

    monkeypatch.setenv("HOME", str(tmp_dir / "HOME"))
    test_home = Path.home()
    
    test_full_path = test_home / test_path_from_home
    Path.mkdir(test_full_path, parents=True)
    os.chdir(test_full_path)
    
    for i in iter(too.POSITIONAL_ARGS):
        too.main(i)

    # expected names created
    assert sorted(os.listdir(test_full_path)) == sorted(list(too.POSITIONAL_ARGS))

    # test link, dir existence
    for i in test_full_path.iterdir():
        assert i.is_symlink()
        assert i.is_dir()

    # read links, verify expected path from HOME
    for i in test_full_path.iterdir():
        too_path = i.readlink()
        too_stem = too_path.relative_to(test_home)
        too_stem_components = str(too_stem).split('/')
        too_stem_anchor = too_stem_components[0]
        assert too_stem_anchor in list(too.POSITIONAL_ARGS.values())

def test_git_repo(tmp_path, monkeypatch):
    tmp_dir = tmp_path
    git_repo_name = "git_repo_dir"
    test_path_from_home = git_repo_name + "/subdir_x/subdir_y"

    monkeypatch.setenv("HOME", str(tmp_dir / "HOME"))
    test_home = Path.home()
    
    test_full_path = test_home / test_path_from_home
    Path.mkdir(test_full_path, parents=True)
    git.Repo.init(test_home / git_repo_name)
    os.chdir(test_full_path)
    
    for i in iter(too.POSITIONAL_ARGS):
        too.main(i)

    # expected names created
    assert sorted(os.listdir(test_full_path)) == sorted(list(too.POSITIONAL_ARGS))

    # test link, dir existence
    for i in test_full_path.iterdir():
        assert i.is_symlink()
        assert i.is_dir()

    # read links, verify expected path from HOME
    for i in test_full_path.iterdir():
        too_path = i.readlink()
        too_stem = too_path.relative_to(test_home)
        too_stem_components = str(too_stem).split('/')
        too_stem_anchor = too_stem_components[0]
        assert too_stem_anchor in list(too.POSITIONAL_ARGS.values())
        assert too_stem_components[1] == git_repo_name
